// src/parser.ts
var RegShakuDirectiveUnderlineSolid = new RegExp("^(?<leadingSpaces>\\s*)(?<content>(-|_)+)\\s*(?<shift><*)\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveUnderlineWavy = new RegExp("^(?<leadingSpaces>\\s*)(?<content>~+)\\s*(?<shift><*)\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveUnderlineDotted = new RegExp("^(?<leadingSpaces>\\s*)(?<content>\\.+)\\s*(?<shift><*)\\s*(?<escape>!?)\\s*$");
var RegShakuAnnotationLine = new RegExp("^(?<leadingSpaces>\\s*)\\[(?<content>.+)\\]\\s*(?<shift><*)\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveHighlightInline = new RegExp("^(\\([a-zA-Z0-9]*\\))+(?<shift><*)(?<escape>!?)$");
var RegShakuDirectiveHighlightInlinePart = new RegExp("\\(\\s*(?<id>[a-zA-Z0-9]*)\\s*\\)", "g");
function isShakuDirectiveHighlightInline(str) {
  var _a, _b, _c;
  const significantLine = str.replace(/\s/g, "");
  const matches = significantLine.match(RegShakuDirectiveHighlightInline);
  return {
    isEscaped: !!((_a = matches == null ? void 0 : matches.groups) == null ? void 0 : _a.escape),
    isHighlightInline: !!matches,
    shift: (_c = (_b = matches == null ? void 0 : matches.groups) == null ? void 0 : _b.shift.length) != null ? _c : 0
  };
}
var RegShakuDirectiveCallout = new RegExp("^(?<leadingSpaces>\\s*)\\^\\s*(?<shift><*)\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveFold = new RegExp("^(?<leadingSpaces>\\s*)@fold(\\s+(?<mark>[a-z\\^]+)?)?\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveHighlight = new RegExp("^(?<leadingSpaces>\\s*)@highlight(\\s+(?<mark>[a-z\\^]+)?)?\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveDim = new RegExp("^(?<leadingSpaces>\\s*)@dim(\\s+(?<mark>[a-z\\^]+)?)?\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveFocus = new RegExp("^(?<leadingSpaces>\\s*)@focus(\\s+(?<mark>[a-z\\^]+)?)?\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveDiff = new RegExp("^(?<leadingSpaces>\\s*)@diff\\s+(?<type>(\\+|\\-))\\s*((?<mark>([a-z\\^]+)?)\\s*)?(?<escape>!?)\\s*$");
var RegShakuDirectiveClass = new RegExp("^(?<leadingSpaces>\\s*)@class\\s+(?<classNames>([a-zA-Z0-9 \\-_]+))\\s*(?<escape>!?)\\s*$");
var RegShakuDirectiveData = new RegExp("^(?<leadingSpaces>\\s*)@data\\s+(?<entries>([a-zA-Z0-9 \\-=_]+))\\s*(?<escape>!?)\\s*$");
var parseLine = (line) => {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i, _j, _k, _l, _m, _n, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _A, _B, _C, _D, _E, _F, _G, _H, _I, _J, _K, _L, _M, _N, _O, _P, _Q, _R, _S, _T, _U, _V, _W, _X, _Y, _Z, __, _$, _aa;
  const matchShakuDirectiveUnderlineSolid = line.match(
    RegShakuDirectiveUnderlineSolid
  );
  if (matchShakuDirectiveUnderlineSolid) {
    return {
      type: "DirectiveUnderline",
      config: {
        isEscaped: !!((_a = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _a.escape),
        offset: ((_c = (_b = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _b.leadingSpaces.length) != null ? _c : 0) - ((_e = (_d = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _d.shift.length) != null ? _e : 0),
        content: (_g = (_f = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _f.content) != null ? _g : "",
        style: "solid"
      }
    };
  }
  const matchShakuDirectiveUnderlineWavy = line.match(
    RegShakuDirectiveUnderlineWavy
  );
  if (matchShakuDirectiveUnderlineWavy) {
    return {
      type: "DirectiveUnderline",
      config: {
        isEscaped: !!((_h = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _h.escape),
        offset: ((_j = (_i = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _i.leadingSpaces.length) != null ? _j : 0) - ((_l = (_k = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _k.shift.length) != null ? _l : 0),
        content: (_n = (_m = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _m.content) != null ? _n : "",
        style: "wavy"
      }
    };
  }
  const matchShakuDirectiveUnderlineDotted = line.match(
    RegShakuDirectiveUnderlineDotted
  );
  if (matchShakuDirectiveUnderlineDotted) {
    return {
      type: "DirectiveUnderline",
      config: {
        isEscaped: !!((_o = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _o.escape),
        offset: ((_q = (_p = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _p.leadingSpaces.length) != null ? _q : 0) - ((_s = (_r = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _r.shift.length) != null ? _s : 0),
        content: (_u = (_t = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _t.content) != null ? _u : "",
        style: "dotted"
      }
    };
  }
  const matchShakuAnnotationLine = line.match(RegShakuAnnotationLine);
  if (matchShakuAnnotationLine) {
    return {
      type: "AnnotationLine",
      config: {
        isEscaped: !!((_v = matchShakuAnnotationLine.groups) == null ? void 0 : _v.escape),
        offset: ((_x = (_w = matchShakuAnnotationLine.groups) == null ? void 0 : _w.leadingSpaces.length) != null ? _x : 0) - ((_z = (_y = matchShakuAnnotationLine.groups) == null ? void 0 : _y.shift.length) != null ? _z : 0),
        content: (_B = (_A = matchShakuAnnotationLine.groups) == null ? void 0 : _A.content) != null ? _B : ""
      }
    };
  }
  const matchShakuDirectiveCallout = line.match(RegShakuDirectiveCallout);
  if (matchShakuDirectiveCallout) {
    return {
      type: "DirectiveCallout",
      config: {
        isEscaped: !!((_C = matchShakuDirectiveCallout.groups) == null ? void 0 : _C.escape),
        offset: ((_E = (_D = matchShakuDirectiveCallout.groups) == null ? void 0 : _D.leadingSpaces.length) != null ? _E : 0) - ((_G = (_F = matchShakuDirectiveCallout.groups) == null ? void 0 : _F.shift.length) != null ? _G : 0)
      }
    };
  }
  const matchShakuDirectiveFold = line.match(RegShakuDirectiveFold);
  if (matchShakuDirectiveFold) {
    const mark = getCanonicalMark(
      (_I = (_H = matchShakuDirectiveFold.groups) == null ? void 0 : _H.mark) != null ? _I : "below"
    );
    if (mark === "start" || mark === "end") {
      return {
        type: "DirectiveFold",
        config: {
          isEscaped: !!((_J = matchShakuDirectiveFold.groups) == null ? void 0 : _J.escape),
          mark
        }
      };
    }
  }
  const matchShakuDirectiveHighlight = line.match(RegShakuDirectiveHighlight);
  if (matchShakuDirectiveHighlight) {
    const mark = getCanonicalMark(
      (_L = (_K = matchShakuDirectiveHighlight.groups) == null ? void 0 : _K.mark) != null ? _L : "below"
    );
    if (mark === "start" || mark === "end" || mark === "below") {
      return {
        type: "DirectiveHighlight",
        config: {
          isEscaped: !!((_M = matchShakuDirectiveHighlight.groups) == null ? void 0 : _M.escape),
          mark
        }
      };
    }
  }
  const matchShakuDirectiveDim = line.match(RegShakuDirectiveDim);
  if (matchShakuDirectiveDim) {
    const mark = getCanonicalMark(
      (_O = (_N = matchShakuDirectiveDim.groups) == null ? void 0 : _N.mark) != null ? _O : "below"
    );
    if (mark === "start" || mark === "end" || mark === "below") {
      return {
        type: "DirectiveDim",
        config: {
          isEscaped: !!((_P = matchShakuDirectiveDim.groups) == null ? void 0 : _P.escape),
          mark
        }
      };
    }
  }
  const matchShakuDirectiveFocus = line.match(RegShakuDirectiveFocus);
  if (matchShakuDirectiveFocus) {
    const mark = getCanonicalMark(
      (_R = (_Q = matchShakuDirectiveFocus.groups) == null ? void 0 : _Q.mark) != null ? _R : "below"
    );
    if (mark === "start" || mark === "end" || mark === "below") {
      return {
        type: "DirectiveFocus",
        config: {
          isEscaped: !!((_S = matchShakuDirectiveFocus.groups) == null ? void 0 : _S.escape),
          mark
        }
      };
    }
  }
  const matchShakuDirectiveDiff = line.match(RegShakuDirectiveDiff);
  if (matchShakuDirectiveDiff) {
    const type = (_T = matchShakuDirectiveDiff.groups) == null ? void 0 : _T.type;
    const mark = getCanonicalMark(
      (_V = (_U = matchShakuDirectiveDiff.groups) == null ? void 0 : _U.mark) != null ? _V : "below"
    );
    if ((type === "+" || type === "-") && (mark === "start" || mark === "end" || mark === "below")) {
      return {
        type: "DirectiveDiff",
        config: {
          isEscaped: !!((_W = matchShakuDirectiveDiff.groups) == null ? void 0 : _W.escape),
          type,
          mark
        }
      };
    }
  }
  const { isHighlightInline, shift, isEscaped } = isShakuDirectiveHighlightInline(line);
  if (isHighlightInline) {
    const parts = [...line.matchAll(RegShakuDirectiveHighlightInlinePart)];
    return {
      type: "DirectiveHighlightInline",
      config: {
        isEscaped,
        parts: parts.map((part) => {
          return {
            offset: part.index - shift,
            length: part[0].length,
            id: part[1] || void 0
          };
        })
      }
    };
  }
  const matchShakuDirectiveClass = line.match(RegShakuDirectiveClass);
  if (matchShakuDirectiveClass) {
    const classNames = (_Y = (_X = matchShakuDirectiveClass.groups) == null ? void 0 : _X.classNames) != null ? _Y : "";
    return {
      type: "DirectiveClass",
      config: {
        isEscaped: !!((_Z = matchShakuDirectiveClass.groups) == null ? void 0 : _Z.escape),
        classNames: classNames.trim()
      }
    };
  }
  const matchShakuDirectiveData = line.match(RegShakuDirectiveData);
  if (matchShakuDirectiveData) {
    const entriesStr = (_$ = (__ = matchShakuDirectiveData.groups) == null ? void 0 : __.entries) != null ? _$ : "";
    const entries = filterNonNull(
      entriesStr.trim().split(/\s+/).map((entry) => {
        const segs = entry.split("=");
        if (segs.length !== 2)
          return null;
        return {
          key: segs[0],
          value: segs[1]
        };
      })
    );
    return {
      type: "DirectiveData",
      config: {
        isEscaped: !!((_aa = matchShakuDirectiveData.groups) == null ? void 0 : _aa.escape),
        entries
      }
    };
  }
  return null;
};
function shouldApplyAnnotation(meta) {
  return /\bannotate\b/.test(meta);
}
function renderComponent(component, options) {
  const useDangerousRawHtml = options == null ? void 0 : options.useDangerousRawHtml;
  switch (component.type) {
    case "ShakuComponentCallout":
      return `<div class="shaku-callout" style="left:${component.config.offset}ch"><span class="shaku-callout-arrow" style="left:${component.config.arrowOffset}ch"></span>${useDangerousRawHtml ? component.config.contents : escapeHtml(component.config.contents)}</div>`;
    case "ShakuComponentUnderline":
      return `<div class="shaku-underline shaku-underline-${component.config.underlineStyle}" style="left:${component.config.offset}ch"><span class="shaku-underline-line" style="left:${component.config.underlineOffset}ch">${component.config.underlineContent}</span>${useDangerousRawHtml ? component.config.contents : escapeHtml(component.config.contents)}</div>`;
    default:
      assertsNever(component);
  }
}
function assertsNever(data) {
  throw new Error("expected never but got: " + data);
}
function escapeHtml(html) {
  return html.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/'/g, "&#039;");
}
function getCanonicalMark(mark) {
  if (mark === "v")
    return "start";
  if (mark === "^")
    return "end";
  return mark;
}
function filterNonNull(arr) {
  return arr.filter((item) => item !== null);
}
export {
  parseLine,
  renderComponent,
  shouldApplyAnnotation
};
